@@ -14,12 +14,12 @@ Test::Config::System - System configuration related unit tests
 
 =head1 VERSION
 
-Version 0.23
+Version 0.31
 
 =cut
 
-our $VERSION     = '0.23';
-our @EXPORT      = qw(check_package check_file check_link check_dir plan diag ok);
+our $VERSION     = '0.31';
+our @EXPORT      = qw(check_package check_file_contents check_link check_file check_dir plan diag ok);
 our $AUTOLOAD;
 
 sub AUTOLOAD {
@@ -40,14 +40,14 @@ tests.
     check_package('less', 'package less');
     check_package('emacs21', 'emacs uninstalled', 1);
     check_link('/etc/alternatives/www-browser', '/usr/bin/w3m');
-    check_file('Test/Config/System.pm', qr/do {local \$\//);
+    check_file_contents('Test/Config/System.pm', qr/do {local \$\//);
 
 =head1 DESCRIPTION
 
-Test::Config::System provides functions to help test system configuration, such as
-installed packages or config files.  It was built for use in a cfengine
-staging environment.  (L<http://www.cfengine.org>.  cfengine is used for
-automating and managing system configuration at large sites).
+Test::Config::System provides functions to help test system configuration,
+such as installed packages or config files.  It was built for use in a
+cfengine staging environment.  (L<http://www.cfengine.org>.  cfengine is used
+for automating and managing system configuration at large sites).
 
 Test::Config::System does not depend on, or interact with cfengine in any way,
 however, and can be used on its own or with another configuration management
@@ -63,18 +63,19 @@ check they preform.
     use Test::Config::System tests => 3;
     # check something we already know:  :)
     check_package('perl', 'perl is installed');
-    check_file('/etc/apt/sources.list', qr/apt-proxy/,
-        'apt-proxy is not being used', 1); # check that the apt-proxy does not
-                                           # appear anywhere in sources.list
+    check_file_contents('/etc/apt/sources.list', qr/apt-proxy/,
+        'apt-proxy is not being used', 1); # the apt-proxy is not
+                                           # anywhere in sources.list
     check_package('vim');      # Make sure we have the one true editor
 
 
 =head1 EXPORT
 
 check_package
-check_file
+check_file_contents
 check_link
 check_dir
+check_file
 
 From Test::Builder::Module:
 
@@ -101,8 +102,9 @@ is the opposite).
 
   - NAME: package name
   - DESC: test name (optional, defaults to package name)
-  - INVERT: invert test (optional, defaults to 0.  Possible values are 0 and 1.)
-  - PACKAGE_MANAGER: package manager (optional and currently ignored, defaults to 'dpkg')
+  - INVERT: invert test (optional.  Possible values are 0 (default) and 1.)
+  - PACKAGE_MANAGER: package manager (optional and currently ignored.
+                                      Defaults to 'dpkg'.)
 
 Examples:
     # Will fail if nethack is not installed
@@ -132,33 +134,39 @@ sub check_package {
     $tb->ok($res == $invert, $testname);
 }
 
-=head2 check_file( FILENAME, REGEX, [DESC, INVERT] )
+=head2 check_file_contents( FILENAME, REGEX, [DESC, INVERT] )
 
-check_file tests that a file's contents match a given regex.  It slurps the
-contents of the given filename, then matches it against the given regex.  If
-the regex matches the contents of the file, ok() is called with a true value,
-causing the test to pass.  If the regex does not match, the test will fail (as
-in check_package, if the test is inverted, the results are inverted as well).
+check_file_contents tests that a file's contents match a given regex.  It
+slurps the contents of the given filename, then matches it against the given
+regex.  If the regex matches the contents of the file, ok() is called with a
+true value, causing the test to pass.  If the regex does not match, the test
+will fail (as in check_package, if the test is inverted, the results are
+inverted as well).
 
-If the given file cannot be opened for reading, check_file will return
-undef, and ok() will not be called.
+If the given file cannot be opened for reading, ok() will be called with a
+false value (unless, as always, invert is true).
 
 Note that the entire file is slurped into memory at once.  This has two
 concequences.  First, large files will eat up RAM.  Second, ^ and $ do not
-work line-by-line, as one might expect.  "\n" can be used in place of ^ or $.
+work line-by-line, as one might expect.  "\n" (or your system's line-end
+sequence) can be used in place of ^ or $ to match the start and end of a
+line.
 
   - FILENAME: file to test
   - REGEX: regex to match (qr//-style regex)
   - DESC: test name (optional, defaults to filename)
-  - INVERT: invert test (optional, defaults to 0.  Possible values are 0 and 1.)
+  - INVERT: invert test (optional.  Possible values are 0 (default) and 1.)
 
 Examples:
-    check_file('/etc/fstab', qr|proc\s+/proc\s+proc\s+defaults\s+0\s+0|, 'proc is in fstab');
-    check_file('/etc/passwd', qr|evilbob:x:\d+:\d+:|, 'evilbob is not in passwd', 1);
+    check_file_contents('/etc/fstab', qr|proc\s+/proc\s+proc\s+defaults\s+0\s+0|,
+        'proc is in fstab');
+    check_file_contents('/etc/passwd', qr|evilbob:x:\d+:\d+:|,
+        'evilbob is not in passwd', 1);
 
 =cut
 
-sub check_file {
+sub check_file_contents {
+    my $tb = Test::Config::System->builder;
     my ($filename, $regex, $testname, $invert, $res);
     $filename   = shift;
     $regex      = shift;
@@ -173,14 +181,16 @@ sub check_file {
     $testname   = (shift || $filename);
     $invert     = (shift || 0);
 
-    $res = (open my $fh, '<', $filename) or carp "Could not open $filename for reading";
-    return undef unless $res;
+    $res = (open my $fh, '<', $filename);
+    unless ($res) {
+#        $tb->diag("Could not open $filename: $!");
+        return $tb->ok($invert ? !$res : $res, $testname);
+    }
 
     my $text = do {local $/; <$fh>};
     close($fh);
     $res = ($text =~ /$regex/);
 
-    my $tb = Test::Config::System->builder;
     $tb->ok($invert ? !$res : $res, $testname);
 }
 
@@ -197,10 +207,10 @@ check_link will return undef.
 
   - FILENAME: filename (path to a symlink)
   - TARGET:   path the link should point to.  optional.  If not specified,
-              check_link will ignore the target, and just verify that the link
-              exists.
+              check_link will ignore the target, and just verify that the
+              link exists.
   - DESC: test name (optional, defaults to filename)
-  - INVERT: invert test (optional, defaults to 0.  Possible values are 0 and 1.)
+  - INVERT: invert test (optional.  Possible values are 0 (default) and 1.)    
 
 Examples:
     check_link('/etc/alternatives/rsh', '/usr/bin/ssh');
@@ -239,37 +249,40 @@ sub check_link {
     $tb->ok($invert ? !$res : $res, $testname);
 }
 
-=head2 check_dir( PATH, [STAT, DESC, INVERT] )
+=head2 check_file( PATH, [STAT, DESC, INVERT] )
 
-check_dir verifies that a directory exists.  Optionally, it can check various
-attributes such as owner, group, or permissions.
+check_file verifies that a directory exists.  Optionally, it can check
+various attributes such as owner, group, or permissions.
 
-  - PATH: path of the directory to check
-  - STAT: a hashref of attributes to check and their desired values.  Valid keys:
+  - PATH: path of the file to check
+  - STAT: a hashref of attributes and their desired values.  Valid keys:
         -uid    owner uid
         -gid    owner gid
         -mode   file permissions
-    Each key is optional, as is the entire hash.  Note that the type need not
-    be specified in -mode; it is added automatically (ie use 0700 instead of
-    040700).
+    Each key is optional, as is the entire hash.  Note that the type need
+    not be specified in -mode; it is added automatically (ie use 0700
+    instead of 010700).
 
   - DESC: test name (optional, defaults to PATH)
-  - INVERT: invert test (optional, defaults to 0.  Possible values are 0 and 1.)
+  - INVERT: invert test (optional.  Possible values are 0 (default) and 1.)
 
 Examples:
-    check_dir('/var/www');
+    check_file('/etc/sudoers', { '-uid' => 0, '-gid' => 0, '-mode' => 0440);
+
+=head2 check_dir( PATH, [STAT, DESC, INVERT] )
+
+check_dir is check_file for directories (they both call the same internal
+sub).  Arguments and calling are exactly the same, the only significant
+difference is that check_dir verifies that PATH is a directory rather than
+a file.
     check_dir('/home/ian', { '-uid' => scalar getpwnam('ian') } );
-    check_dir('/root/', { '-uid' => 0, '-gid' => 0, '-mode' => 0700 }, '/root/ has sane permissions');
-    check_dir('/home/evilbob', { }, 'Evilbobs homedir does not exist', 1);
+    check_dir('/root/', { '-uid' => 0, '-gid' => 0, '-mode' => 0700 },
+        '/root/ has sane permissions');
+    check_dir('/home/evilbob', { }, 'Evilbobs homedir does not exist',1);
 
 =cut
 
 sub check_dir {
-    my %tbl = ( '-uid'  => 4,
-                '-gid'  => 5,
-                '-mode' => 2,
-              );
-
     my ($path, $stat, $testname, $invert,$res);
     $path = shift;
     if (!$path) {
@@ -282,13 +295,49 @@ sub check_dir {
     $invert   = (shift || 0);
 
     my $tb = Test::Config::System->builder;
+    unless (-d $path) {
+        return $tb->ok($invert ? 1 : 0, $testname);
+    }
 
-    $res = 0;
-    $res = 1 if (-d $path);     # is it a directory?
+    $res = _check_path(040000, $path, $stat);
+    $tb->ok($invert ? !$res : $res, $testname);
+}
 
-    if ($res && ref($stat) eq 'HASH') { # path is a dir and $stat was supplied
+sub check_file {
+    my ($path, $stat, $testname, $invert,$res);
+    $path = shift;
+    if (!$path) {
+        carp "path required";
+        return undef;
+    }
+    $stat     = (shift || { });
+    $testname = (shift || $path);
+    $invert   = (shift || 0);
+
+    my $tb = Test::Config::System->builder;
+    unless (-f $path) {
+        return $tb->ok($invert ? 1 : 0, $testname);
+    }
+
+    $res = _check_path(0100000, $path, $stat);
+    $tb->ok($invert ? !$res : $res, $testname);
+
+}
+
+sub _check_path {
+    my %tbl = ( '-uid'  => 4,
+                '-gid'  => 5,
+                '-mode' => 2,
+              );
+    ## Internal function, we can assume args are checked by the caller.
+    my ($type, $path, $stat) = @_;
+    my $tb = Test::Config::System->builder;
+
+    my $res = 1;
+
+    if (ref($stat) eq 'HASH') {
         my @attrs = stat($path);
-        $attrs[2] -= 040000; # Already know the type, only matching perms
+        $attrs[2] -= $type; # Already know the type, only matching perms
         ## Loop through each pair or until $res is false
         while ((my ($key, $val) = each(%$stat)) && $res) {
             ## Look up the key's position in the list returned from stat(),
@@ -296,12 +345,12 @@ sub check_dir {
             if (exists($tbl{$key})) {
                 $res = ($attrs[$tbl{$key}] == $val) ? 1 : 0;
             }
-            $tb->diag("required: $key->$val, actual: $key->" . $attrs[$tbl{$key}]) unless $res;
+            $tb->diag("required: $key->$val, actual: $key->"
+                . $attrs[$tbl{$key}]) unless $res;
         }
     }
-    $tb->ok($invert ? !$res : $res, $testname);
+    return $res; # caller will do the ok()
 }
-#    $val = oct($val) if $val =~ /^0/;
 
 =head1 AUTHOR
 
@@ -311,9 +360,10 @@ Ian Kilgore, C<< <iank at cpan.org> >>
 
 =over 4
 
-=item * Never call check_package or check_file with untrusted input.
+=item * Never call check_package or check_file_contents with untrusted input.
 
-check_package shells out to dpkg, and check_file uses an arbitrary regexp.
+check_package shells out to dpkg, and check_file_contents uses an arbitrary
+regexp.
 
 =item * check_package currently only supports dpkg
 
@@ -357,7 +407,8 @@ L<http://search.cpan.org/dist/Test-Config-System>
 
 =head1 ACKNOWLEDGEMENTS
 
-Many thanks to [naikonta] at perlmonks, for reviewing the module before release.
+Many thanks to [naikonta] at perlmonks, for reviewing the module before
+release.
 
 =head1 COPYRIGHT & LICENSE
 
